#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

#include <math.h>

#define ARRAY_SIZE(x) (sizeof (x) / sizeof (x[0]))

DWORD WINAPI
FPUTest(LPVOID param)
{
        int c, test = (INT)param;
        double result, wanted;

        for (c = 0; c < 5; c++)
        {
                if (test == 0)
                {
                        double x;
                        int i;

                        result = 0.0;
                        x = 2.0;

                        wanted = 100000000.0;
                        for (i = 0; i < 50000000; i++)
                        {
                                result += x;
                        }
                }
                else if (test == 1)
                {
                        double x;
                        int i;

                        x = 1.0;
                        result = 0.0;

                        wanted = 50000000.0;
                        for (i = 0; i < 50000000; i++)
                        {
                                result += x;
                        }
                }
                else
                {
                        printf("FPUTest(): Unexpected param: 0x%x\n", (UINT)param);
                        return 0; /* failure */
                }

                if (result != wanted)
                {
                        printf("FPUTest(): test #%d failed: result = %lf, wanted = %lf\n", test + 1, result, wanted);
                        return 0; /* failure */
                }
        }

        return 1; /* success */
}

int
main()
{
        INT numThreads;
        HANDLE hThread[2];
        BOOL testSuccessful = TRUE;

        printf("FPU state saving test started.\n");

        /* Create threads */
        for (numThreads = 0; numThreads < ARRAY_SIZE(hThread); numThreads++)
        {
                DWORD dwDummy;
                hThread[numThreads] = CreateThread(NULL, 0, FPUTest, (LPVOID)numThreads, 0, &dwDummy);
                if (hThread[numThreads] == NULL)
                {
                        printf("CreateThread() failed: last error = 0x%x\n", (UINT)GetLastError());
                        while (--numThreads >= 0)
                        {
                                TerminateThread(hThread[numThreads], 0);
                                CloseHandle(hThread[numThreads]);
                        }
                        return EXIT_FAILURE;
                }
        }

        /* Wait for all threads to terminate, get their exit code
         * and close their handles.
         */
        while (--numThreads >= 0)
        {
                DWORD exitCode = 0;
                WaitForSingleObject(hThread[numThreads], INFINITE);
                if (!GetExitCodeThread(hThread[numThreads], &exitCode))
                {
                        printf("GetThreadExitCode() failed: last error = 0x%x\n", (UINT)GetLastError());
                        testSuccessful = FALSE;
                }
                else
                        testSuccessful = (testSuccessful && exitCode);
                CloseHandle(hThread[numThreads]);
        }

        printf("FPU state saving test %s\n", testSuccessful ? "succeeded." : "FAILED!");
        return (testSuccessful ? EXIT_SUCCESS : EXIT_FAILURE);
}

